    { "whois.corenic.net",	"utf-8",	"-C UTF-8" },
    { "whois.cat",	"utf-8",	"-C UTF-8" },
    { "whois.museum",	"utf-8",	"-C UTF-8" },
    { "whois.nic.br",	"iso-8859-1",	NULL },
    { "whois.cira.ca",	"iso-8859-1",	NULL },
    { "whois.nic.ch",	"utf-8",	NULL },
    { "whois.nic.cl",	"iso-8859-1",	NULL },
    { "whois.cnnic.cn",	"utf-8",	NULL },
    { "cwhois.cnnic.cn",	"utf-8",	NULL },
    { "whois.nic.cz",	"utf-8",	NULL },
    { "whois.denic.de",	"utf-8",	NULL },
    { "whois.enum.denic.de",	"utf-8",	NULL },
    { "whois.dk-hostmaster.dk",	"utf-8",	"--charset=utf-8" },
    { "whois.eenet.ee",	"iso-8859-1",	NULL },
    { "whois.eu",	"utf-8",	NULL },
    { "whois.ficora.fi",	"iso-8859-1",	NULL },
    { "whois.nic.fr",	"iso-8859-1",	NULL },
    { "whois.hkdnr.net.hk",	"utf-8",	NULL },
    { "whois.nic.hu",	"iso-8859-1",	NULL },
    { "whois.isnic.is",	"iso-8859-1",	NULL },
    { "whois.jprs.jp",	"iso-2022-jp",	NULL },
    { "whois.nic.ad.jp",	"iso-2022-jp",	NULL },
    { "whois.nic.or.kr",	"euc-kr",	NULL },
    { "whois.nic.li",	"utf-8",	NULL },
    { "whois.domreg.lt",	"utf-8",	NULL },
    { "whois.dns.lu",	"iso-8859-1",	NULL },
    { "whois.nic.mu",	"utf-8",	NULL },
    { "whois.norid.no",	"iso-8859-1",	NULL },
    { "whois.nic.nu",	"utf-8",	NULL },
    { "whois.dns.pt",	"iso-8859-1",	NULL },
    { "whois.nic-se.se",	"utf-8",	NULL },
    { "whois.nic.tr",	"utf-8",	NULL },
    { "whois.twnic.net",	"utf-8",	NULL },
    { "whois.net.ua",	"koi8-u",	NULL },
    { "whois.nic.org.uy",	"utf-8",	NULL },
